<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:redirect="http://xml.apache.org/xalan/redirect"
 extension-element-prefixes="redirect">

   <xsl:output
    method="xml"
    indent="yes"
    encoding="UTF-8"/>

	<xsl:include href="xradar://stylesheets/radar/statics/process/master-merge.xsl"/>

	<xsl:param    name="input-report-file" />
    <xsl:variable name="jcoverage-report-doc" select="document($input-report-file)"/>


	<xsl:variable
    	name="report-source"
    	select="'JCoverage'" />
	<xsl:variable
    	name="report-source-link"
    	select="'http://www.jcoverage.com/'"/>


   <xsl:template
    	match="Stats">
		<xsl:copy>
			<xsl:apply-templates select="@*|node()"/>
			<xsl:if test="not(test-metrics)">
				<test-metrics>
					<xsl:apply-templates
						select="$jcoverage-report-doc/coverage/packages" mode="test-stats">
						<xsl:with-param
							name="package-name">
							<xsl:value-of select="../@name" />
						</xsl:with-param>
						<xsl:with-param
							name="source-statements">
							<xsl:value-of select="ncss" />
						</xsl:with-param>
					</xsl:apply-templates>
				</test-metrics>
			</xsl:if>
		</xsl:copy>
   </xsl:template>

   <xsl:template
    	match="test-metrics">
      <xsl:copy>
		 <xsl:apply-templates select="@*|node()"/>
			 <xsl:apply-templates
	          	select="$jcoverage-report-doc/coverage/packages" mode="test-stats">
				  <xsl:with-param
		             name="package-name">
		             	<xsl:value-of select="../../@name" />
		          </xsl:with-param>
		          <xsl:with-param
		             name="source-statements">
		             	<xsl:value-of select="../ncss" />
		          </xsl:with-param>
	         </xsl:apply-templates>
      </xsl:copy>
   </xsl:template>


   <xsl:template
    	match="packages" mode="test-stats">
		<xsl:param name="package-name" />
		<xsl:param name="source-statements" />

		<line-coverage><xsl:value-of select="package[@name = $package-name]/@line"/></line-coverage>
		<source-statements-covered><xsl:value-of select="package[@name = $package-name]/@line * $source-statements"/></source-statements-covered>
   </xsl:template>

   <xsl:template name="package-addition"/>


   <xsl:template
    	match="Class">
		<xsl:variable name="class-name" select="normalize-space(node())"/>
		<xsl:variable name="current-jcoverage-class" select="$jcoverage-report-doc/coverage/class[@name=$class-name]"/>
		<xsl:copy>
			<xsl:apply-templates select="@*|node()">
				<xsl:with-param name="current-jcoverage-class" select="$current-jcoverage-class"/>
			</xsl:apply-templates>

			<xsl:if test="../../Stats/test-metrics">
				<test-metrics>
					<line-coverage><xsl:value-of select="$current-jcoverage-class/line/@rate"/></line-coverage>
				</test-metrics>
			</xsl:if>
		</xsl:copy>
   </xsl:template>

   <xsl:template
    	match="functions">
    	<xsl:param name="current-jcoverage-class"/>
  		<xsl:copy>
			<xsl:apply-templates select="@*|node()">
				<xsl:with-param name="current-jcoverage-class" select="$current-jcoverage-class"/>
			</xsl:apply-templates>
		</xsl:copy>
	</xsl:template>

   <xsl:template
    	match="function">
    		<xsl:param name="current-jcoverage-class"/>
			<xsl:variable name="class-name" select="normalize-space(../../node())"/>
			<xsl:variable name="class-name-without-package" select="substring-before(../../@sourceFile,'.java')"/>
			<xsl:variable name="method-name-with-params" select="normalize-space(substring-after(name,concat($class-name,'.')))"/>
			<xsl:variable name="method-name" select="substring-before($method-name-with-params,'(')"/>


			<xsl:copy>
				<xsl:apply-templates select="@*|node()"/>
		 		<xsl:if test="($current-jcoverage-class/methods/method[starts-with(@nameAndSignature,$method-name)]) or ($method-name = $class-name-without-package)">
			 		<test-metrics>
	 					<line-coverage>
	 						<xsl:for-each select="$current-jcoverage-class/methods/method[starts-with(@nameAndSignature,concat($method-name,'('))]">
	 							<xsl:call-template name="if-equal-parameters-return-value">
	 								<xsl:with-param name="orig-param-set" select="substring-before(substring-after($method-name-with-params,'('),')')"/>
	 								<xsl:with-param name="orig-param-set-mod" select="substring-before(substring-after($method-name-with-params,'('),')')"/>
	 								<xsl:with-param name="jcoverage-param-set" select="substring-before(substring-after(@nameAndSignature,'('),')')"/>
	 								<xsl:with-param name="converted-param-set" select="''"/>
	 								<xsl:with-param name="value" select="line/@rate" />
	 							</xsl:call-template>
	 						</xsl:for-each>
	 						<xsl:if test="$method-name = $class-name-without-package">
		 						<xsl:for-each select="$current-jcoverage-class/methods/method[contains(substring(@nameAndSignature,1,6),'init')]">
		 							<xsl:call-template name="if-equal-parameters-return-value">
		 								<xsl:with-param name="orig-param-set" select="substring-before(substring-after($method-name-with-params,'('),')')"/>
		 								<xsl:with-param name="orig-param-set-mod" select="substring-before(substring-after($method-name-with-params,'('),')')"/>
		 								<xsl:with-param name="jcoverage-param-set" select="substring-before(substring-after(@nameAndSignature,'('),')')"/>
		 								<xsl:with-param name="converted-param-set" select="''"/>
		 								<xsl:with-param name="value" select="line/@rate" />
		 							</xsl:call-template>
		 						</xsl:for-each>
		 					</xsl:if>
	 					</line-coverage>
					</test-metrics>
				</xsl:if>
			</xsl:copy>
   </xsl:template>



	<xsl:template name="if-equal-parameters-return-value">
		<xsl:param name="orig-param-set"/>
		<xsl:param name="orig-param-set-mod"/>
		<xsl:param name="jcoverage-param-set"/>
		<xsl:param name="converted-param-set"/>
		<xsl:param name="value" select="line/@rate"/>

		<xsl:choose>
			<xsl:when test="starts-with($jcoverage-param-set,'I')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'I')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'int,')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($jcoverage-param-set,'[I')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'[I')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'int[],')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:when test="starts-with($jcoverage-param-set,'S')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'S')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'short,')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($jcoverage-param-set,'[S')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'[S')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'short[],')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:when test="starts-with($jcoverage-param-set,'B')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'B')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'byte,')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($jcoverage-param-set,'[B')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'[B')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'byte[],')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:when test="starts-with($jcoverage-param-set,'D')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'D')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'double,')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($jcoverage-param-set,'[D')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'[D')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'double[],')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:when test="starts-with($jcoverage-param-set,'F')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'F')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'float,')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($jcoverage-param-set,'[F')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'[F')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'float[],')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:when test="starts-with($jcoverage-param-set,'Z')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'Z')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'boolean,')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($jcoverage-param-set,'[Z')">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,'[Z')"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,'boolean[],')"/>
					<xsl:with-param name="value" select="$value"/>
				</xsl:call-template>
			</xsl:when>


			<xsl:when test="(starts-with($jcoverage-param-set,'L'))
								and
							(
								(contains($orig-param-set-mod,','))
									and
								(contains(substring-before($jcoverage-param-set,';'),substring-before($orig-param-set-mod,',')) )
							)">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,';')"/>
					<xsl:with-param name="value" select="$value"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,substring-before($orig-param-set-mod,','),',')"/>
				</xsl:call-template>
			</xsl:when>


			<xsl:when test="(starts-with($jcoverage-param-set,'L'))
								and
							(
								(not(contains($orig-param-set-mod,',')))
									and
								(contains(substring-before($jcoverage-param-set,';'),$orig-param-set-mod) )
							)">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,';')"/>
					<xsl:with-param name="value" select="$value"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,$orig-param-set-mod,',')"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:when test="(starts-with($jcoverage-param-set,'[L'))
								and
							(
								(contains($orig-param-set-mod,','))
									and
								(contains(substring-before($orig-param-set-mod,','),'[]'))
									and
								(contains(substring-before($jcoverage-param-set,';'),substring-before($orig-param-set-mod,'[],')))
							)">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,';')"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="value" select="$value"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,substring-before($orig-param-set-mod,','),',')"/>
				</xsl:call-template>
			</xsl:when>


			<xsl:when test="(starts-with($jcoverage-param-set,'[L'))
								and
							(
								(not(contains($orig-param-set-mod,',')))
									and
								(contains($orig-param-set-mod,'[]'))
									and
								( contains(substring-before($jcoverage-param-set,';'),substring-before($orig-param-set-mod,'[]')))
							)">
				<xsl:call-template name="if-equal-parameters-return-value">
					<xsl:with-param name="orig-param-set" select="$orig-param-set"/>
					<xsl:with-param name="jcoverage-param-set" select="substring-after($jcoverage-param-set,';')"/>
					<xsl:with-param name="orig-param-set-mod" select="substring-after($orig-param-set-mod,',')"/>
					<xsl:with-param name="value" select="$value"/>
					<xsl:with-param name="converted-param-set" select="concat($converted-param-set,$orig-param-set-mod,',')"/>
				</xsl:call-template>
			</xsl:when>

			<xsl:otherwise>
				<xsl:if test="(concat($orig-param-set,',') = $converted-param-set)
									or
							  ($orig-param-set = $converted-param-set)">
					<xsl:value-of select="$value"/>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>


</xsl:stylesheet>
